﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using System.IO;

using Pac.Character;
using Pac.Constants;
using Pac.Resources;
using Pac.App;

namespace Pac.Scene 
{
    public class SummaryScene : BaseScene
    {
        Pacman pac;
        bool saved = false;
        string playerName = "";

        KeyboardState prevState;

        private static SummaryScene instance;

        public static SummaryScene getInstance(Pacman pac)
        {
            if (instance == null)
            {
                instance = new SummaryScene();
            }
            instance.pac = pac;
            return instance;
        }

        private SummaryScene()
        {
        }

        public override BaseScene Update(GameTime gameTime)
        {
            prevState = keyboardState;
            keyboardState = Keyboard.GetState();
            if (!saved && pac.score > GameConstants.highScore)
            {
                foreach (Keys k in keyboardState.GetPressedKeys())
                {
                    if (prevState.IsKeyUp(k))
                    {
                        if (k.Equals(Keys.Enter))
                        {
                            GameConstants.highScore = pac.score;
                            GameConstants.playerName = playerName;
                            saveHighScore();
                            saved = true;
                        }
                        else if (k.Equals(Keys.Space) && playerName.Length < 3)
                        {
                            playerName = playerName.Insert(playerName.Length, " ");
                        }
                        else if (k.Equals(Keys.Back) && playerName.Length > 0)
                        {
                            playerName = playerName.Remove(playerName.Length - 1, 1);
                        }
                        else
                        {
                            if(playerName.Length + k.ToString().Length <= 3 )
                                playerName += k.ToString();
                        }
                    }
                }
            }
            else if (keyboardState.IsKeyDown(Keys.Space))
            {
                int lvl = 0;
                saved = false;
                lvl = PacmanScene.getInstance().level;
                if (pac.lives > 0)
                {
                    lvl++;
                    GameConstants.initMap();
                    PacmanScene.getInstance().initialize(lvl);
                    PacmanScene.getInstance().pac.score = pac.score;
                    return PacmanScene.getInstance();
                }
                else
                {
                    return MenuScene.getInstance();
                }
            }
            else if (keyboardState.IsKeyDown(Keys.Escape))
            {
                return MenuScene.getInstance();
            }
            return this;
        }

        private void saveHighScore()
        {
            StreamWriter w = null;
            try
            {
                w = new StreamWriter("temp");
                w.Write(GameConstants.playerName+","+GameConstants.highScore + "");
                w.Close();
                FileSecurity.EncryptFile("temp", GameConstants.SAVE_FILE);
            }
            catch (FileNotFoundException e)
            {
            }
        }

        public override void Draw(GameTime gameTime, SpriteBatch spriteBatch)
        {
            spriteBatch.Begin(SpriteBlendMode.AlphaBlend);
            spriteBatch.Draw(GameTexture.levelBackgroundSprite, new Rectangle(toX(0), toY(0), toX(800), toY(600)), Color.White);
            if (!saved && pac.score > GameConstants.highScore)
            {
                spriteBatch.DrawString(GameTexture.spriteFont, "NEW HIGHSCORE !!", new Vector2(toX(310), toY(200)), Color.YellowGreen);
                spriteBatch.DrawString(GameTexture.spriteFont, "Enter your name :", new Vector2(toX(350), toY(300)), Color.YellowGreen);
                spriteBatch.DrawString(GameTexture.spriteFont, playerName, new Vector2(toX(350), toY(370)), Color.Red);
            }
            else if (pac.lives == 0)
            {
                spriteBatch.Draw(GameTexture.gameOverSprite, new Rectangle(toX(250), toY(200), toX(300), toY(100)), Color.White);
                spriteBatch.DrawString(GameTexture.spriteFont, "Press Space to go to main menu...", new Vector2(toX(310), toY(400)), Color.YellowGreen);
            }
            else
            {
                spriteBatch.Draw(GameTexture.levelCompleteSprite, new Rectangle(toX(250), toY(200), toX(300), toY(100)), Color.White);
                spriteBatch.DrawString(GameTexture.spriteFont, "Press Space to start next level...", new Vector2(toX(300), toY(400)), Color.YellowGreen);
            }
            spriteBatch.DrawString(GameTexture.spriteFont,"Score : "+pac.score,new Vector2(toX(350),toY(100)),Color.Yellow);
            spriteBatch.End();   
        }
    }
}
